library crud;

import 'package:flutter/foundation.dart';
import 'package:restified/http.dart';
import 'package:rxdart/rxdart.dart';

class GetBloc<T> {
  final String endpoint;
  final Serializable<T> model;

  final BehaviorSubject<List<T>> _all = BehaviorSubject<List<T>>();
  final BehaviorSubject<T> _single = BehaviorSubject<T>();

  GetBloc({this.endpoint, this.model});

  getAll() async {
    _all.sink.add(null);
    List<T> response = await Http().getAll('$endpoint', model);
    _all.sink.add(response);
  }

  getById(int id) async {
    _single.sink.add(null);
    T m = await Http().getSingle('$endpoint/$id', model);
    _single.sink.add(m);
  }

  change(T model) {
    _single.sink.add(model);
  }

  dispose() {
    _all.close();
    _single.close();
  }

  BehaviorSubject<List<T>> get all => _all;

  BehaviorSubject<T> get single => _single;
}

abstract class ManipulationBloc<T> {
  final BehaviorSubject<T> _response = BehaviorSubject<T>();
  final BehaviorSubject<bool> _loading = BehaviorSubject<bool>();

  dispose() {
    _response.close();
    _loading.close();
  }

  BehaviorSubject<T> get response => _response;

  BehaviorSubject<bool> get loading => _loading;
}

class UpdateBloc extends ManipulationBloc<UpdateResponse> {
  final String endpoint;

  UpdateBloc({@required this.endpoint});

  update(int id, Map model) async {
    _loading.sink.add(true);
    UpdateResponse resp = await Http().put('$endpoint/$id', model);
    _response.sink.add(resp);
    _loading.sink.add(false);
  }
}

class InsertBloc extends ManipulationBloc<InsertResponse> {
  final String endpoint;

  InsertBloc({@required this.endpoint});

  insert(Map model) async {
    _loading.sink.add(true);
    InsertResponse resp = await Http().post('$endpoint', model);
    _response.sink.add(resp);
    _loading.sink.add(false);
  }
}
